var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var os_1 = require("os");
var Q = require("q");
var vssCoreContracts = require("vso-node-api/interfaces/common/VSSInterfaces");
var trace = require("../../lib/trace");
var witBase = require("./default");
function getCommand(args) {
    return new WorkItemCreate(args);
}
exports.getCommand = getCommand;
var WorkItemCreate = (function (_super) {
    __extends(WorkItemCreate, _super);
    function WorkItemCreate() {
        _super.apply(this, arguments);
    }
    WorkItemCreate.prototype.getHelpArgs = function () {
        return ["workItemType", "title", "assignedTo", "description", "project"];
    };
    WorkItemCreate.prototype.exec = function () {
        var witapi = this.webApi.getQWorkItemTrackingApi();
        return Q.all([
            this.commandArgs.workItemType.val(),
            this.commandArgs.assignedTo.val(true),
            this.commandArgs.title.val(),
            this.commandArgs.description.val(true),
            this.commandArgs.project.val()
        ]).spread(function (wiType, assignedTo, title, description, project) {
            var patchDoc = [];
            patchDoc.push({
                op: vssCoreContracts.Operation.Add,
                path: "/fields/System.Title",
                value: title,
                from: null
            });
            if (assignedTo) {
                patchDoc.push({
                    op: vssCoreContracts.Operation.Add,
                    path: "/fields/System.AssignedTo",
                    value: assignedTo,
                    from: null
                });
            }
            if (description) {
                patchDoc.push({
                    op: vssCoreContracts.Operation.Add,
                    path: "/fields/System.Description",
                    value: description,
                    from: null
                });
            }
            return witapi.updateWorkItemTemplate(null, patchDoc, project, wiType);
        });
    };
    WorkItemCreate.prototype.friendlyOutput = function (workItem) {
        if (!workItem) {
            throw new Error("no results");
        }
        trace.success(os_1.EOL + "Created Work Item @ " + workItem.id + os_1.EOL);
        trace.info("id:          " + workItem.id);
        trace.info("rev:         " + workItem.rev);
        trace.info("type:        " + workItem.fields["System.WorkItemType"]);
        trace.info("state:       " + workItem.fields["System.State"]);
        trace.info("title:       " + workItem.fields["System.Title"]);
        trace.info("assigned to: " + workItem.fields["System.AssignedTo"]);
    };
    return WorkItemCreate;
})(witBase.WorkItemBase);
exports.WorkItemCreate = WorkItemCreate;
